//+------------------------------------------------------------------+
//|                             OzFx Expert Advisor for MetaTrader 4 |
//|                                                      version 1.0 |
//|                                          original idea from OzFx |
//|                                           EA conversion by Azmel |
//+------------------------------------------------------------------+
//| INSTRUCTIONS                                                     |
//| To be used on any chart with D1 timeframe.                       |
//+------------------------------------------------------------------+
//| SETTINGS                                                         |
//| AccountType=0 for Standard (1 Lot) Accounts                      |
//| AccountType=1 for Mini (0.1 Lot) Accounts                        |
//| AccountType=2 for Micro (0.01 Lot) Accounts                      |
//| AccountType=3 for InterbankFX Nano (0.01 Lot) Accounts           |
//+------------------------------------------------------------------+
//
//  200SMA Filter added by Matt Ebersviller

#include <stdlib.mqh>

//+------------------------------------------------------------------+
//| External Variables                                               |
//+------------------------------------------------------------------+

extern int    Magic           = 887453;
extern double Lots            = 0.01;
extern bool   MoneyManagement = true;
extern double UseFreeMargin   = 80;
extern int    AccountType     = 3;
extern int    Leverage        = 100;
extern bool   UseFilter_200SMA = true;

//+------------------------------------------------------------------+
//| Internal variables                                               |
//+------------------------------------------------------------------+

int    PricePerPip;
double DecimalCorrection;
int    DecimalPlaces;
double MaxLots;

bool   OKToTrade;
int    i;
int    OpenOrders;

double ACSignalCurrent;
double ACSignalPrevious;
double StochasticSignal;
bool   BuyFlag;
bool   SellFlag;
bool   StopLossAdjust;
bool   StartUp=true;

int    ticket1;
int    ticket2;
int    ticket3;
int    ticket4;
int    ticket5;

int    TP1 = 50;
int    TP2 = 100;
int    TP3 = 150;
int    TP4 = 200;

int    SL  = 100;

int    slippage = 5;

int    StochasticUpper = 50;
int    StochasticLower = 50;

double EntryPrice;

//+------------------------------------------------------------------+
//| Main code                                                        |
//+------------------------------------------------------------------+

int start()
{

   //+---------------------------------------------------------------+
   //| Account Type Initiasation                                     |
   //+---------------------------------------------------------------+


   if (AccountType==0)
   {
      PricePerPip       = 10;
      DecimalCorrection = 0.5;
      DecimalPlaces     = 0;
   }
   
   if (AccountType==1)
   { 
      PricePerPip       = 10;
      DecimalCorrection = 0.05;
      DecimalPlaces     = 1;
   }
   
   if (AccountType==2)
   {
      PricePerPip       = 10;
      DecimalCorrection = 0.005;
      DecimalPlaces     = 2;
   }
   
   if (AccountType==3)
   {
      PricePerPip       = 1;
      DecimalCorrection = 0.005;
      DecimalPlaces     = 2;
   }
   
   //+---------------------------------------------------------------+
   //| Money Management Routine                                      |
   //+---------------------------------------------------------------+

   if (MoneyManagement)
   {
      if (UseFreeMargin > 100)
      {
         Comment("ERROR: UseFreeMargin value too large.");
         OKToTrade=false;
      }
      else
      {
         if (UseFreeMargin < 1)
         {
            Comment("ERROR: UseFreeMargin value too small.");
            OKToTrade=false;
         }
         else
         {
            Lots=NormalizeDouble(((AccountFreeMargin()*(UseFreeMargin/5)*Leverage)/
               (Ask*PricePerPip*1000000))-DecimalCorrection,DecimalPlaces);
            OKToTrade=true;
         }
      }
   }
   else
   {
      MaxLots=NormalizeDouble(((AccountFreeMargin()*20*Leverage)/
         (Ask*PricePerPip*1000000))-DecimalCorrection,DecimalPlaces);
      if (Lots > MaxLots)
      {
         Comment("ERROR: Insufficient Free Margin.\nLower Lots value or use MoneyManagement.");
         OKToTrade=false;
      }
      else
      {
         OKToTrade=true;
      }
   }

   //+---------------------------------------------------------------+
   //| Time Frame Check                                              |
   //+---------------------------------------------------------------+
   
   if (Period() != 1440)
   {
      OKToTrade=false;
      Comment("Wrong Time Frame. Please switch to D1.");
   }
   
   //+---------------------------------------------------------------+
   //| Start-Up Routine (To Prevent Immediate Entry During Start-Up) |
   //+---------------------------------------------------------------+

   if (StartUp)
   {
      if (Hour()==0)
      {
         OKToTrade = true;
         StartUp   = false;
      }
      else
      {
         OKToTrade = false;
         StartUp   = true;
         Comment("Staring Up Delay. Awaiting Next Candle.");
      }
   }

   //+---------------------------------------------------------------+
   //| Main Trading Routine                                          |
   //+---------------------------------------------------------------+

   if (OKToTrade)
   {

      //+------------------------------------------------------------+
      //| Signal & Indicators                                        |
      //+------------------------------------------------------------+

      ACSignalCurrent  = iAC(NULL,0,0);
      ACSignalPrevious = iAC(NULL,0,1);
      StochasticSignal = iStochastic(NULL,0,5,3,3,MODE_SMA,0,MODE_MAIN,0);
      
      if (UseFilter_200SMA) {
         double MA_Filter = iMA(NULL,0,200,0,MODE_SMA,PRICE_OPEN,0);
      }
      
      Comment("AC Current  = ",ACSignalCurrent
           ,"\nAC Previous = ",ACSignalPrevious
           ,"\nStochastic  = ",StochasticSignal
           ,"\n\nLots = ",Lots);

      //+------------------------------------------------------------+
      //| Check Condition To Open Long                               |
      //+------------------------------------------------------------+

      if (ACSignalCurrent>0 && ACSignalPrevious<0 && StochasticSignal>StochasticUpper && BuyFlag==false)
      {
         if (UseFilter_200SMA == false || (UseFilter_200SMA == true && MA_Filter > OrderOpenPrice() )) 
         {
            OrderSelect(ticket1,SELECT_BY_TICKET);
            if (OrderType()==OP_SELL)
            {
               OrderClose(ticket1,OrderLots(),Ask,slippage,CLR_NONE);
            }
      
            OrderSelect(ticket2,SELECT_BY_TICKET);
            if (OrderType()==OP_SELL)
            {
               OrderClose(ticket2,OrderLots(),Ask,slippage,CLR_NONE);
            }

            OrderSelect(ticket3,SELECT_BY_TICKET);
            if (OrderType()==OP_SELL)
            {
               OrderClose(ticket3,OrderLots(),Ask,slippage,CLR_NONE);
            }

            OrderSelect(ticket4,SELECT_BY_TICKET);
            if (OrderType()==OP_SELL)
            {
               OrderClose(ticket4,OrderLots(),Ask,slippage,CLR_NONE);
            }

            OrderSelect(ticket5,SELECT_BY_TICKET);
            if (OrderType()==OP_SELL)
            {
               OrderClose(ticket5,OrderLots(),Ask,slippage,CLR_NONE);
            }

            SellFlag=false;
   
            ticket1=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,Ask-SL*Point,Ask+TP1*Point,"OzFx-1B",Magic,CLR_NONE);
            ticket2=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,Ask-SL*Point,Ask+TP2*Point,"OzFx-2B",Magic,CLR_NONE);
            ticket3=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,Ask-SL*Point,Ask+TP3*Point,"OzFx-3B",Magic,CLR_NONE);
            ticket4=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,Ask-SL*Point,Ask+TP4*Point,"OzFx-4B",Magic,CLR_NONE);
            ticket5=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,Ask-SL*Point,0,"OzFx-5B",Magic,CLR_NONE);

            EntryPrice=Ask;         
            BuyFlag=true;
            StopLossAdjust=false;
         }
      }

      //+------------------------------------------------------------+
      //| Check Condition To Open Short                              |
      //+------------------------------------------------------------+

      if (ACSignalCurrent<0 && ACSignalPrevious>0 && StochasticSignal<StochasticLower && SellFlag==false)
      {
         if (UseFilter_200SMA == false || (UseFilter_200SMA == true && MA_Filter < OrderOpenPrice() ))
         {
            OrderSelect(ticket1,SELECT_BY_TICKET);
            if (OrderType()==OP_BUY)
            {
               OrderClose(ticket1,OrderLots(),Bid,slippage,CLR_NONE);
            }
         
            OrderSelect(ticket2,SELECT_BY_TICKET);
            if (OrderType()==OP_BUY)
            {
               OrderClose(ticket2,OrderLots(),Bid,slippage,CLR_NONE);
            }

            OrderSelect(ticket3,SELECT_BY_TICKET);
            if (OrderType()==OP_BUY)
            {
               OrderClose(ticket3,OrderLots(),Bid,slippage,CLR_NONE);
            }

            OrderSelect(ticket4,SELECT_BY_TICKET);
            if (OrderType()==OP_BUY)
            {
               OrderClose(ticket4,OrderLots(),Bid,slippage,CLR_NONE);
            }

            OrderSelect(ticket5,SELECT_BY_TICKET);
            if (OrderType()==OP_BUY)
            {
               OrderClose(ticket5,OrderLots(),Bid,slippage,CLR_NONE);
            }

            BuyFlag=false;

            ticket1=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,Bid+SL*Point,Bid-TP1*Point,"OzFx-1S",Magic,CLR_NONE);
            ticket2=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,Bid+SL*Point,Bid-TP2*Point,"OzFx-2S",Magic,CLR_NONE);
            ticket3=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,Bid+SL*Point,Bid-TP3*Point,"OzFx-3S",Magic,CLR_NONE);
            ticket4=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,Bid+SL*Point,Bid-TP4*Point,"OzFx-4S",Magic,CLR_NONE);
            ticket5=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,Bid+SL*Point,0,"OzFx-5S",Magic,CLR_NONE);

            EntryPrice=Bid;
            SellFlag=true;
            StopLossAdjust=false;
         }
      }

      //+------------------------------------------------------------+
      //| If Trade one hit TakeProfit, move StopLoss to BreakEven    |
      //+------------------------------------------------------------+

      if (StopLossAdjust==false)
      {
         if ((BuyFlag && Bid>=EntryPrice+TP1*Point) || (SellFlag && Ask<=EntryPrice-TP1*Point))
         {
            OrderSelect(ticket2,SELECT_BY_TICKET);
            OrderModify(ticket2,OrderOpenPrice(),EntryPrice,OrderTakeProfit(),0,CLR_NONE);

            OrderSelect(ticket3,SELECT_BY_TICKET);
            OrderModify(ticket3,OrderOpenPrice(),EntryPrice,OrderTakeProfit(),0,CLR_NONE);

            OrderSelect(ticket4,SELECT_BY_TICKET);
            OrderModify(ticket4,OrderOpenPrice(),EntryPrice,OrderTakeProfit(),0,CLR_NONE);

            OrderSelect(ticket5,SELECT_BY_TICKET);
            OrderModify(ticket5,OrderOpenPrice(),EntryPrice,OrderTakeProfit(),0,CLR_NONE);

            StopLossAdjust=true;
         }
      }
   }

   //+---------------------------------------------------------------+
   //| Reset sequence if all open orders have been closed            |
   //+---------------------------------------------------------------+

   OpenOrders=0;
   for(i=0;i<OrdersTotal();i++)   
   {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
	  if (OrderSymbol()==Symbol() && OrderMagicNumber() == Magic)
	  {				
	  	  OpenOrders++;
	  }
   } 

   if (OpenOrders==0)
   {
      BuyFlag=false;
      SellFlag=false;
      StopLossAdjust=false;
   }

   return(0);
}